package com.xxfc.platform.vehicle.pojo;

import com.github.wxiaoqi.security.common.vo.PageParam;
import com.xxfc.platform.vehicle.entity.VehiclePlatCata;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.persistence.Column;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;

import static com.github.wxiaoqi.security.common.constant.CommonConstants.YMR_SLASH_FORMATTER;

@Data
public class RentVehicleBookDTO extends PageParam {
    //根据车型、时间、距离，门店，预定车辆
    @ApiModelProperty("车型id")
    private Integer modelId;

    @ApiModelProperty("品牌id")
    private Integer brandId;

    @ApiModelProperty("型号id")
    private Integer categoryId;

    @ApiModelProperty("商品类型 1--房车；2--机车；3--游艇")
    private Integer goodsType;

    /**
     * 预定目标日期(开始)
     */
    @ApiModelProperty("预定目标日期(开始)")
    private String bookStartDate;

    /**
     * 预定目标日期(结束)
     */
    @ApiModelProperty("预定目标日期(结束)")
    private String bookEndDate;

    /**
     * 预定目标日期(开始)
     */
    @ApiModelProperty("预定目标时间(开始)")
    private String bookStartDateTime;

    /**
     * 预定目标日期(结束)
     */
    @ApiModelProperty("预定目标时间(结束)")
    private String bookEndDateTime;


    /**
     * 提车地点
     */
    @ApiModelProperty("提车地点")
    private String liftAddr;

    /**
     * 提车公司
     */
    @ApiModelProperty("提车公司")
    private Integer liftCompany;

    /**
     * 还车分公司
     */
    @ApiModelProperty("还车分公司")
    private Integer retCompany;

    /**
     * 目的地
     */
    @ApiModelProperty("目的地")
    private String destination;

    /**
     *  用户名
     */
    @ApiModelProperty("用户名")
    private String userName;

    //使用人姓名
    private String vehicleUsername;
    //使用人电话
    private String vehicleUserPhone;

    @ApiModelProperty("订单号")
    private String orderNo;

    @ApiModelProperty("指定车牌")
    String numberPlate;

    @ApiModelProperty("指定预约状态")
    Integer status;

    /**
     * 随车物品id以及数量 List<Map<id,数量>>
     */
    @ApiModelProperty("随车物品id以及数量 List<Map<id,数量>>")
    Map<Integer,Integer> selectedAccItem;

    @ApiModelProperty("停靠公司id")
    Integer parkBranchCompanyId;

    @ApiModelProperty("开始公司id")
    Integer startCompanyId;

    @ApiModelProperty("结束公司id")
    Integer endCompanyId;

    @ApiModelProperty("扩展开始时间")
    String startDateExtend;

    @ApiModelProperty("扩展结束时间")
    String endDateExtend;

    @ApiModelProperty("会员免费类型")
    Integer memberUse;

    @ApiModelProperty("车辆id")
    String vehicleId;

    /**
     * 用途 1 租车
     */
    @ApiModelProperty(hidden = true)
    Integer useType = 1;

    /**
     * 是否不检查时间字节上的冲突
     */
    @ApiModelProperty(hidden = true)
    Boolean yearNo4Where;

    /**
     * 是否根据预订记录(相交叉)查询
     * 原（大于预约结束时间，小于预约开始时间的预约记录）
     * 相交叉（大于预约开始时间，小于预约结束时间的预约记录）
     */
    @ApiModelProperty(hidden = true)
    Boolean recordIntersection;

    @ApiModelProperty("城市")
    Integer addrCity;
    @ApiModelProperty("经度")
    Double lon;
    @ApiModelProperty("纬度")
    Double lat;

    @ApiModelProperty("分类逗号分割")
    String catasStr;
    @ApiModelProperty(value = "分类列表", hidden = true)
    Map<Integer, List<VehiclePlatCata>> catas;
    @ApiModelProperty(value = "分类列表", hidden = true)
    Map<Integer, List<VehiclePlatCata>> catas2;

    public void setBookStartDate(String bookStartDate) {
        this.bookStartDate = bookStartDate;
        //首次设置
        if(null == this.startDateExtend) {
            this.startDateExtend = YMR_SLASH_FORMATTER.format(LocalDate.parse(bookStartDate, YMR_SLASH_FORMATTER).minusDays(1L));
        }
    }

    public void setBookEndDate(String bookEndDate) {
        this.bookEndDate = bookEndDate;
        //首次设置
        if(null == this.endDateExtend) {
            this.endDateExtend = YMR_SLASH_FORMATTER.format(LocalDate.parse(bookEndDate, YMR_SLASH_FORMATTER).plusDays(1L));
        }
    }
}